@@ -29,10 +29,12 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef WIN32
+#ifdef LIBUSBMUXD_STATIC
+  #define USBMUXD_API
+#elif defined(_WIN32)
   #define USBMUXD_API __declspec( dllexport )
 #else
-  #ifdef HAVE_FVISIBILITY
+  #if __GNUC__ >= 4
     #define USBMUXD_API __attribute__((visibility("default")))
   #else
     #define USBMUXD_API
@@ -61,7 +63,6 @@
 #else
 #include <sys/socket.h>
 #include <arpa/inet.h>
-#include <pthread.h>
 #if defined(HAVE_PROGRAM_INVOCATION_SHORT_NAME) && !defined(HAVE_PROGRAM_INVOCATION_SHORT_NAME_ERRNO_H)
 extern char *program_invocation_short_name;
 #endif
@@ -74,6 +75,7 @@ extern int _NSGetExecutablePath(char* buf, uint32_t* bufsize);
 #ifdef HAVE_INOTIFY
 #include <sys/inotify.h>
 #include <sys/select.h>
+#include <pthread.h>
 #define EVENT_SIZE  (sizeof (struct inotify_event))
 #define EVENT_BUF_LEN (1024 * (EVENT_SIZE + 16))
 #define USBMUXD_DIRNAME "/var/run"
@@ -103,11 +105,11 @@ static char *prog_name = NULL;
 // usbmuxd protocol
 #include "usbmuxd-proto.h"
 // socket utility functions
-#include "socket.h"
+#include <libimobiledevice-glue/socket.h>
 // misc utility functions
-#include "collection.h"
+#include <libimobiledevice-glue/collection.h>
 // threads
-#include "thread.h"
+#include <libimobiledevice-glue/thread.h>
 
 static int libusbmuxd_debug = 0;
 #ifndef PACKAGE
@@ -158,6 +160,7 @@ static usbmuxd_device_info_t *devices_find(uint32_t handle)
  */
 static int connect_usbmuxd_socket()
 {
+	int res = -1;
 	char *usbmuxd_socket_addr = getenv("USBMUXD_SOCKET_ADDRESS");
 	if (usbmuxd_socket_addr) {
 		if (strncmp(usbmuxd_socket_addr, "UNIX:", 5) == 0) {
@@ -165,7 +168,11 @@ static int connect_usbmuxd_socket()
 			/* not supported, ignore */
 #else
 			if (usbmuxd_socket_addr[5] != '\0') {
-				return socket_connect_unix(usbmuxd_socket_addr+5);
+				res = socket_connect_unix(usbmuxd_socket_addr+5);
+				if (res < 0) {
+					res = -errno;
+				}
+				return res;
 			}
 #endif
 		} else {
@@ -194,11 +201,14 @@ static int connect_usbmuxd_socket()
 					connect_addr[p - usbmuxd_socket_addr] = '\0';
 				}
 				if (connect_addr && *connect_addr != '\0') {
-					int res = socket_connect(connect_addr, port);
+					res = socket_connect(connect_addr, port);
 #ifdef HAVE_INOTIFY
 					use_inotify = 0;
 #endif
 					free(connect_addr);
+					if (res < 0) {
+						res = -errno;
+					}
 					return res;
 				}
 				free(connect_addr);
@@ -206,10 +216,14 @@ static int connect_usbmuxd_socket()
 		}
 	}
 #if defined(WIN32) || defined(__CYGWIN__)
-	return socket_connect("127.0.0.1", USBMUXD_SOCKET_PORT);
+	res = socket_connect("127.0.0.1", USBMUXD_SOCKET_PORT);
 #else
-	return socket_connect_unix(USBMUXD_SOCKET_FILE);
+	res = socket_connect_unix(USBMUXD_SOCKET_FILE);
 #endif
+	if (res < 0) {
+		res = -errno;
+	}
+	return res;
 }
 
 static void sanitize_udid(usbmuxd_device_info_t *devinfo)
@@ -270,13 +284,11 @@ static usbmuxd_device_info_t *device_info_from_plist(plist_t props)
 				devinfo->conn_type = CONNECTION_TYPE_NETWORK;
 				n = plist_dict_get_item(props, "NetworkAddress");
 				if (n && plist_get_node_type(n) == PLIST_DATA) {
-					char *netaddr = NULL;
 					uint64_t addr_len = 0;
-					plist_get_data_val(n, &netaddr, &addr_len);
+					const char *netaddr = plist_get_data_ptr(n, &addr_len);
 					if (netaddr && addr_len > 0 && addr_len < sizeof(devinfo->conn_data)) {
 						memcpy(devinfo->conn_data, netaddr, addr_len);
 					}
-					free(netaddr);
 				}
 			} else {
 				LIBUSBMUXD_ERROR("%s: Unexpected ConnectionType '%s'\n", __func__, strval);
@@ -340,7 +352,9 @@ static int receive_packet(int sfd, struct usbmuxd_header *header, void **payload
 			LIBUSBMUXD_DEBUG(1, "%s: Error receiving packet: %s\n", __func__, strerror(-recv_len));
 		}
 		return recv_len;
-	} else if ((size_t)recv_len < sizeof(hdr)) {
+	}
+
+	if ((size_t)recv_len < sizeof(hdr)) {
 		LIBUSBMUXD_DEBUG(1, "%s: Received packet is too small, got %d bytes!\n", __func__, recv_len);
 		return recv_len;
 	}
@@ -500,7 +514,9 @@ static int usbmuxd_get_result(int sfd, uint32_t tag, uint32_t *result, void **re
 		}
 		free(res);
 		return ret;
-	} else if (hdr.message == MESSAGE_PLIST) {
+	}
+
+	if (hdr.message == MESSAGE_PLIST) {
 		if (!result_plist) {
 			LIBUSBMUXD_DEBUG(1, "%s: MESSAGE_PLIST result but result_plist pointer is NULL!\n", __func__);
 			return -1;
@@ -685,11 +701,11 @@ static void get_prog_name()
 	size_t r = fread(tmpbuf, 1, 512, f);
 	if (r > 0) {
 		char *p = tmpbuf;
-		while ((p-tmpbuf < r) && (*p != '(') && (*p != '\0')) p++;
+		while (((size_t)(p-tmpbuf) < r) && (*p != '(') && (*p != '\0')) p++;
 		if (*p == '(') {
 			p++;
 			char *pname = p;
-			while ((p-tmpbuf < r) && (*p != ')') && (*p != '\0')) p++;
+			while (((size_t)(p-tmpbuf) < r) && (*p != ')') && (*p != '\0')) p++;
 			if (*p == ')') {
 				*p = '\0';
 				prog_name = strdup(pname);
@@ -807,7 +823,7 @@ static int send_pair_record_packet(int sfd, uint32_t tag, const char* msgtype, c
 	if (device_id > 0) {
 		plist_dict_set_item(plist, "DeviceID", plist_new_uint(device_id));
 	}
-	
+
 	res = send_plist_packet(sfd, tag, plist);
 	plist_free(plist);
 
@@ -900,7 +916,7 @@ static int usbmuxd_listen_inotify()
 		return sfd;
 
 	sfd = -1;
-	inot_fd = inotify_init ();
+	inot_fd = inotify_init1(IN_CLOEXEC);
 	if (inot_fd < 0) {
 		LIBUSBMUXD_DEBUG(1, "%s: Failed to setup inotify\n", __func__);
 		return -2;
@@ -923,7 +939,9 @@ static int usbmuxd_listen_inotify()
 		int r = pselect(inot_fd+1, &rfds, NULL, NULL, &tv, NULL);
 		if (r < 0) {
 			break;
-		} else if (r == 0) {
+		}
+
+		if (r == 0) {
 			continue;
 		}
 
@@ -939,9 +957,9 @@ static int usbmuxd_listen_inotify()
 
 			/* check that it's ours */
 			if (pevent->mask & IN_CREATE &&
-			    pevent->len &&
-			    pevent->name[0] != 0 &&
-			    strcmp(pevent->name, USBMUXD_SOCKET_NAME) == 0) {
+				pevent->len &&
+				pevent->name[0] != 0 &&
+				strcmp(pevent->name, USBMUXD_SOCKET_NAME) == 0) {
 				/* retry if usbmuxd isn't ready yet */
 				int retry = 10;
 				while (--retry >= 0) {
@@ -1112,7 +1130,7 @@ static void *device_monitor(void *data)
 		while (running) {
 			int res = get_next_event(listenfd);
 			if (res < 0) {
-			    break;
+				break;
 			}
 		}
 
@@ -1138,31 +1156,31 @@ static void init_listeners(void)
 	mutex_init(&listener_mutex);
 }
 
-USBMUXD_API int usbmuxd_events_subscribe(usbmuxd_subscription_context_t *ctx, usbmuxd_event_cb_t callback, void *user_data)
+int usbmuxd_events_subscribe(usbmuxd_subscription_context_t *context, usbmuxd_event_cb_t callback, void *user_data)
 {
-	if (!ctx || !callback) {
+	if (!context || !callback) {
 		return -EINVAL;
 	}
 
 	thread_once(&listener_init_once, init_listeners);
 
 	mutex_lock(&listener_mutex);
-	*ctx = malloc(sizeof(struct usbmuxd_subscription_context));
-	if (!*ctx) {
+	*context = malloc(sizeof(struct usbmuxd_subscription_context));
+	if (!*context) {
 		mutex_unlock(&listener_mutex);
 		LIBUSBMUXD_ERROR("ERROR: %s: malloc failed\n", __func__);
 		return -ENOMEM;
 	}
-	(*ctx)->callback = callback;
-	(*ctx)->user_data = user_data;
+	(*context)->callback = callback;
+	(*context)->user_data = user_data;
 
-	collection_add(&listeners, *ctx);
+	collection_add(&listeners, *context);
 
 	if (devmon == THREAD_T_NULL || !thread_alive(devmon)) {
 		mutex_unlock(&listener_mutex);
 		int res = thread_new(&devmon, device_monitor, NULL);
 		if (res != 0) {
-			free(*ctx);
+			free(*context);
 			LIBUSBMUXD_DEBUG(1, "%s: ERROR: Could not start device watcher thread!\n", __func__);
 			return res;
 		}
@@ -1173,7 +1191,7 @@ USBMUXD_API int usbmuxd_events_subscribe(usbmuxd_subscription_context_t *ctx, us
 				usbmuxd_event_t ev;
 				ev.event = UE_DEVICE_ADD;
 				memcpy(&ev.device, dev, sizeof(usbmuxd_device_info_t));
-				(*ctx)->callback(&ev, (*ctx)->user_data);
+				(*context)->callback(&ev, (*context)->user_data);
 			}
 		} ENDFOREACH
 		mutex_unlock(&listener_mutex);
@@ -1182,26 +1200,26 @@ USBMUXD_API int usbmuxd_events_subscribe(usbmuxd_subscription_context_t *ctx, us
 	return 0;
 }
 
-USBMUXD_API int usbmuxd_events_unsubscribe(usbmuxd_subscription_context_t ctx)
+int usbmuxd_events_unsubscribe(usbmuxd_subscription_context_t context)
 {
 	int ret = 0;
 	int num = 0;
 
-	if (!ctx) {
+	if (!context) {
 		return -EINVAL;
 	}
 
 	mutex_lock(&listener_mutex);
-	if (collection_remove(&listeners, ctx) == 0) {
+	if (collection_remove(&listeners, context) == 0) {
 		FOREACH(usbmuxd_device_info_t *dev, &devices) {
 			if (dev) {
 				usbmuxd_event_t ev;
 				ev.event = UE_DEVICE_REMOVE;
 				memcpy(&ev.device, dev, sizeof(usbmuxd_device_info_t));
-				(ctx)->callback(&ev, (ctx)->user_data);
+				(context)->callback(&ev, (context)->user_data);
 			}
 		} ENDFOREACH
-		free(ctx);
+		free(context);
 	}
 	num = collection_count(&listeners);
 	mutex_unlock(&listener_mutex);
@@ -1229,7 +1247,7 @@ USBMUXD_API int usbmuxd_events_unsubscribe(usbmuxd_subscription_context_t ctx)
 	return ret;
 }
 
-USBMUXD_API int usbmuxd_subscribe(usbmuxd_event_cb_t callback, void *user_data)
+int usbmuxd_subscribe(usbmuxd_event_cb_t callback, void *user_data)
 {
 	if (!callback) {
 		return -EINVAL;
@@ -1242,14 +1260,14 @@ USBMUXD_API int usbmuxd_subscribe(usbmuxd_event_cb_t callback, void *user_data)
 	return usbmuxd_events_subscribe(&event_ctx, callback, user_data);
 }
 
-USBMUXD_API int usbmuxd_unsubscribe()
+int usbmuxd_unsubscribe(void)
 {
 	int res = usbmuxd_events_unsubscribe(event_ctx);
 	event_ctx = NULL;
 	return res;
 }
 
-USBMUXD_API int usbmuxd_get_device_list(usbmuxd_device_info_t **device_list)
+int usbmuxd_get_device_list(usbmuxd_device_info_t **device_list)
 {
 	int sfd;
 	int tag;
@@ -1362,7 +1380,7 @@ retry:
 			free(payload);
 		} else {
 			// we _should_ have all of them now.
-			// or perhaps an error occured.
+			// or perhaps an error occurred.
 			break;
 		}
 	}
@@ -1390,7 +1408,7 @@ got_device_list:
 	return dev_cnt;
 }
 
-USBMUXD_API int usbmuxd_device_list_free(usbmuxd_device_info_t **device_list)
+int usbmuxd_device_list_free(usbmuxd_device_info_t **device_list)
 {
 	if (device_list) {
 		free(*device_list);
@@ -1398,7 +1416,7 @@ USBMUXD_API int usbmuxd_device_list_free(usbmuxd_device_info_t **device_list)
 	return 0;
 }
 
-USBMUXD_API int usbmuxd_get_device_by_udid(const char *udid, usbmuxd_device_info_t *device)
+int usbmuxd_get_device_by_udid(const char *udid, usbmuxd_device_info_t *device)
 {
 	usbmuxd_device_info_t *dev_list = NULL;
 	usbmuxd_device_info_t *dev = NULL;
@@ -1441,7 +1459,7 @@ USBMUXD_API int usbmuxd_get_device_by_udid(const char *udid, usbmuxd_device_info
 	return result;
 }
 
-USBMUXD_API int usbmuxd_get_device(const char *udid, usbmuxd_device_info_t *device, enum usbmux_lookup_options options)
+int usbmuxd_get_device(const char *udid, usbmuxd_device_info_t *device, enum usbmux_lookup_options options)
 {
 	usbmuxd_device_info_t *dev_list = NULL;
 	usbmuxd_device_info_t *dev_network = NULL;
@@ -1466,7 +1484,8 @@ USBMUXD_API int usbmuxd_get_device(const char *udid, usbmuxd_device_info_t *devi
 			if ((options & DEVICE_LOOKUP_USBMUX) && (dev_list[i].conn_type == CONNECTION_TYPE_USB)) {
 				dev_usbmuxd = &dev_list[i];
 				break;
-			} else if ((options & DEVICE_LOOKUP_NETWORK) && (dev_list[i].conn_type == CONNECTION_TYPE_NETWORK)) {
+			}
+			if ((options & DEVICE_LOOKUP_NETWORK) && (dev_list[i].conn_type == CONNECTION_TYPE_NETWORK)) {
 				dev_network = &dev_list[i];
 				break;
 			}
@@ -1505,7 +1524,7 @@ USBMUXD_API int usbmuxd_get_device(const char *udid, usbmuxd_device_info_t *devi
 	return result;
 }
 
-USBMUXD_API int usbmuxd_connect(const uint32_t handle, const unsigned short port)
+int usbmuxd_connect(const uint32_t handle, const unsigned short port)
 {
 	int sfd;
 	int tag;
@@ -1515,13 +1534,12 @@ USBMUXD_API int usbmuxd_connect(const uint32_t handle, const unsigned short port
 retry:
 	sfd = connect_usbmuxd_socket();
 	if (sfd < 0) {
-		result = errno;
-		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(result));
-		return -result;
+		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(-sfd));
+		return sfd;
 	}
 
 	tag = ++use_tag;
-	if (send_connect_packet(sfd, tag, (uint32_t)handle, (uint16_t)port) <= 0) {
+	if (send_connect_packet(sfd, tag, handle, (uint16_t)port) <= 0) {
 		LIBUSBMUXD_DEBUG(1, "%s: Error sending connect message!\n", __func__);
 	} else {
 		// read ACK
@@ -1558,26 +1576,27 @@ retry:
 	return -result;
 }
 
-USBMUXD_API int usbmuxd_disconnect(int sfd)
+int usbmuxd_disconnect(int sfd)
 {
 	return socket_close(sfd);
 }
 
-USBMUXD_API int usbmuxd_send(int sfd, const char *data, uint32_t len, uint32_t *sent_bytes)
+int usbmuxd_send(int sfd, const char *data, uint32_t len, uint32_t *sent_bytes)
 {
 	int num_sent;
 
 	if (sfd < 0) {
 		return -EINVAL;
 	}
-	
+
 	num_sent = socket_send(sfd, (void*)data, len);
 	if (num_sent < 0) {
 		*sent_bytes = 0;
-		num_sent = errno;
-		LIBUSBMUXD_DEBUG(1, "%s: Error %d when sending: %s\n", __func__, num_sent, strerror(num_sent));
+		LIBUSBMUXD_DEBUG(1, "%s: Error %d when sending: %s\n", __func__, -num_sent, strerror(-num_sent));
 		return -num_sent;
-	} else if ((uint32_t)num_sent < len) {
+	}
+
+	if ((uint32_t)num_sent < len) {
 		LIBUSBMUXD_DEBUG(1, "%s: Warning: Did not send enough (only %d of %d)\n", __func__, num_sent, len);
 	}
 
@@ -1586,7 +1605,7 @@ USBMUXD_API int usbmuxd_send(int sfd, const char *data, uint32_t len, uint32_t *
 	return 0;
 }
 
-USBMUXD_API int usbmuxd_recv_timeout(int sfd, char *data, uint32_t len, uint32_t *recv_bytes, unsigned int timeout)
+int usbmuxd_recv_timeout(int sfd, char *data, uint32_t len, uint32_t *recv_bytes, unsigned int timeout)
 {
 	int num_recv = socket_receive_timeout(sfd, (void*)data, len, 0, timeout);
 	if (num_recv < 0) {
@@ -1599,12 +1618,12 @@ USBMUXD_API int usbmuxd_recv_timeout(int sfd, char *data, uint32_t len, uint32_t
 	return 0;
 }
 
-USBMUXD_API int usbmuxd_recv(int sfd, char *data, uint32_t len, uint32_t *recv_bytes)
+int usbmuxd_recv(int sfd, char *data, uint32_t len, uint32_t *recv_bytes)
 {
 	return usbmuxd_recv_timeout(sfd, data, len, recv_bytes, 5000);
 }
 
-USBMUXD_API int usbmuxd_read_buid(char **buid)
+int usbmuxd_read_buid(char **buid)
 {
 	int sfd;
 	int tag;
@@ -1617,7 +1636,7 @@ USBMUXD_API int usbmuxd_read_buid(char **buid)
 
 	sfd = connect_usbmuxd_socket();
 	if (sfd < 0) {
-		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(errno));
+		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(-sfd));
 		return sfd;
 	}
 
@@ -1645,7 +1664,7 @@ USBMUXD_API int usbmuxd_read_buid(char **buid)
 	return ret;
 }
 
-USBMUXD_API int usbmuxd_read_pair_record(const char* record_id, char **record_data, uint32_t *record_size)
+int usbmuxd_read_pair_record(const char* record_id, char **record_data, uint32_t *record_size)
 {
 	int sfd;
 	int tag;
@@ -1659,8 +1678,7 @@ USBMUXD_API int usbmuxd_read_pair_record(const char* record_id, char **record_da
 
 	sfd = connect_usbmuxd_socket();
 	if (sfd < 0) {
-		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n",
-				__func__, strerror(errno));
+		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(-sfd));
 		return sfd;
 	}
 
@@ -1693,7 +1711,7 @@ USBMUXD_API int usbmuxd_read_pair_record(const char* record_id, char **record_da
 	return ret;
 }
 
-USBMUXD_API int usbmuxd_save_pair_record_with_device_id(const char* record_id, uint32_t device_id, const char *record_data, uint32_t record_size)
+int usbmuxd_save_pair_record_with_device_id(const char* record_id, uint32_t device_id, const char *record_data, uint32_t record_size)
 {
 	int sfd;
 	int tag;
@@ -1705,8 +1723,7 @@ USBMUXD_API int usbmuxd_save_pair_record_with_device_id(const char* record_id, u
 
 	sfd = connect_usbmuxd_socket();
 	if (sfd < 0) {
-		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n",
-				__func__, strerror(errno));
+		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(-sfd));
 		return sfd;
 	}
 
@@ -1732,12 +1749,12 @@ USBMUXD_API int usbmuxd_save_pair_record_with_device_id(const char* record_id, u
 	return ret;
 }
 
-USBMUXD_API int usbmuxd_save_pair_record(const char* record_id, const char *record_data, uint32_t record_size)
+int usbmuxd_save_pair_record(const char* record_id, const char *record_data, uint32_t record_size)
 {
 	return usbmuxd_save_pair_record_with_device_id(record_id, 0, record_data, record_size);
 }
 
-USBMUXD_API int usbmuxd_delete_pair_record(const char* record_id)
+int usbmuxd_delete_pair_record(const char* record_id)
 {
 	int sfd;
 	int tag;
@@ -1749,8 +1766,7 @@ USBMUXD_API int usbmuxd_delete_pair_record(const char* record_id)
 
 	sfd = connect_usbmuxd_socket();
 	if (sfd < 0) {
-		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n",
-				__func__, strerror(errno));
+		LIBUSBMUXD_DEBUG(1, "%s: Error: Connection to usbmuxd failed: %s\n", __func__, strerror(-sfd));
 		return sfd;
 	}
 
@@ -1774,16 +1790,23 @@ USBMUXD_API int usbmuxd_delete_pair_record(const char* record_id)
 	return ret;
 }
 
-USBMUXD_API void libusbmuxd_set_use_inotify(int set)
+void libusbmuxd_set_use_inotify(int set)
 {
 #ifdef HAVE_INOTIFY
 	use_inotify = set;
 #endif
-	return;
 }
 
-USBMUXD_API void libusbmuxd_set_debug_level(int level)
+void libusbmuxd_set_debug_level(int level)
 {
 	libusbmuxd_debug = level;
 	socket_set_verbose(level);
 }
+
+const char* libusbmuxd_version()
+{
+#ifndef PACKAGE_VERSION
+#error PACKAGE_VERSION is not defined
+#endif
+	return PACKAGE_VERSION;
+}
