@@ -37,8 +37,8 @@
 #include <errno.h>
 #include <getopt.h>
 #ifdef WIN32
-#include <windows.h>
 #include <winsock2.h>
+#include <windows.h>
 #else
 #include <sys/select.h>
 #include <sys/socket.h>
@@ -48,7 +48,7 @@
 #endif
 
 #include "usbmuxd.h"
-#include "socket.h"
+#include <libimobiledevice-glue/socket.h>
 
 static int debug_level = 0;
 
@@ -197,7 +197,8 @@ int main(int argc, char **argv)
             if (dev_list[i].conn_type == CONNECTION_TYPE_USB && (lookup_opts & DEVICE_LOOKUP_USBMUX)) {
                 dev = &(dev_list[i]);
                 break;
-            } else if (dev_list[i].conn_type == CONNECTION_TYPE_NETWORK && (lookup_opts & DEVICE_LOOKUP_NETWORK)) {
+            }
+            if (dev_list[i].conn_type == CONNECTION_TYPE_NETWORK && (lookup_opts & DEVICE_LOOKUP_NETWORK)) {
                 dev = &(dev_list[i]);
                 break;
             }
@@ -215,14 +216,14 @@ int main(int argc, char **argv)
         unsigned char saddr_[32];
         memset(saddr_, '\0', sizeof(saddr_));
         struct sockaddr* saddr = (struct sockaddr*)&saddr_[0];
-        if (((char*)dev->conn_data)[1] == 0x02) { // AF_INET
+        if (dev->conn_data[1] == 0x02) { // AF_INET
             saddr->sa_family = AF_INET;
-            memcpy(&saddr->sa_data[0], (char*)dev->conn_data+2, 14);
+            memcpy(&saddr->sa_data[0], (uint8_t*)dev->conn_data+2, 14);
         }
-        else if (((char*)dev->conn_data)[1] == 0x1E) { //AF_INET6 (bsd)
+        else if (dev->conn_data[1] == 0x1E) { //AF_INET6 (bsd)
 #ifdef AF_INET6
             saddr->sa_family = AF_INET6;
-            memcpy(&saddr->sa_data[0], (char*)dev->conn_data+2, 26);
+            memcpy(&saddr->sa_data[0], (uint8_t*)dev->conn_data+2, 26);
 #else
             fprintf(stderr, "ERROR: Got an IPv6 address but this system doesn't support IPv6\n");
             free(dev_list);
@@ -230,7 +231,7 @@ int main(int argc, char **argv)
 #endif
         }
         else {
-            fprintf(stderr, "Unsupported address family 0x%02x\n", ((char*)dev->conn_data)[1]);
+            fprintf(stderr, "Unsupported address family 0x%02x\n", dev->conn_data[1]);
             free(dev_list);
             return 1;
         }
